
from aomame.microsoft import MicrosoftTranslator
from aomame.systran import SystranTranslator
from aomame.google import GoogleTranslator
#from aomame.modernmt import ModernmtTranslator
#from aomame.deepl import DeeplTranslator
